<?php

$analyseTableAlign = NULL;
$analyseTableX = 0;
$analyseTableY = 0;
$analyseTableHead = 0;

$analyseTableKeepMap = array();
$analyseTableKeepIndex = 1;
$analyseTableKeepToken = $KeepToken;

function analyseTableKeep($x) {
  # Keep preserves a string from being processed by wiki markups
  global $analyseTableKeepToken,$analyseTableKeepMap,$analyseTableKeepIndex;
  
  $x = preg_replace("/$analyseTableKeepToken(\\d.*?)$analyseTableKeepToken/e", "\$analyseTableKeepMap['\$1']", $x);
  $analyseTableKeepIndex++;
  $analyseTableKeepMap[$analyseTableKeepIndex]=$x;
  return $analyseTableKeepToken.$analyseTableKeepIndex.$analyseTableKeepToken;
}

function analyseTables(&$doc) {
  global $analyseTableKeepToken,$analyseTableKeepMap;
  return preg_replace
    ("/$analyseTableKeepToken(\\d.*?)$analyseTableKeepToken/e", "\$analyseTableKeepMap['\$1']",
     preg_replace_callback
     ('|<advtabular.*?>(.*?)</advtabular>|s', "analyseAdvancedTable",
     preg_replace_callback
     ('|(<tabular.*?>)(.*?</tabular>)|s', "analyseTable",
      $doc)));

}

function analyseTable($tmatches) {
  // first, get all cell data
  global $analyseTableAlign, $analyseTableY, $analyseTableHead;

  $table = $tmatches[0];
  $tableX = 0;
  if (preg_match("|<tabular[^>]+class=['\"]long['\"]|", $table)) {
    $tableX = 1;
  }
  $cells = array();
  preg_match_all("|<row.*?>.*?</row>|s", $table, $rmatches, PREG_SET_ORDER);
  $cols = 0;
  foreach($rmatches as $rm) {
    $x = 0;
    preg_match_all("|<cell(.*?)>.*?</cell>|s", $rm[0], $cmatches, PREG_SET_ORDER);
    foreach($cmatches as $cm) {
      $align = 'left';
      if (preg_match("|align=['\"](.+?)['\"]|", $cm[1], $amatches)) {
	$align = $amatches[1];
      }
      $wrap = 1;
      if (preg_match("|wrap=['\"]no['\"]|", $cm[1], $dummy)) {
	$wrap = 0;
      } elseif ($tableX) {
	$align = 'X';
      }
      $colspan = 1;
      if (preg_match("|colspan=['\"](.+?)['\"]|", $cm[1], $csmatches)) {
	$colspan = $csmatches[1];
      }
      if ($colspan == 1) {
	$cells[$x][$align]++;
	$cells[$x]['wrap'] += $wrap;
      }
      $x += $colspan;
      if ($cols < $x) { $cols = $x; }
    }
  }
  $colalign = array();
  $preamble = '';
  for($i = 0; $i < $cols; $i++) {
    $lnum = $cells[$i]['left'];
    $cnum = $cells[$i]['center'];
    $rnum = $cells[$i]['right'];
    $xnum = $cells[$i]['X'];
    $max = max($lnum,$rnum,$cnum,$xnum);
    $align = 'left'; // default
    if ($lnum == $max) $align = 'left';
    if ($rnum == $max) $align = 'right';
    if ($cnum == $max) $align = 'center';
    if ($tableX && $cells[$i]['wrap'] > 0) $align = 'X';
    $colalign[$i] = $align;
    if ($cells[$i]['wrap'] == 0) {
      $colalign['wrap'][$i] = 0;
      $preamble .= substr($align, 0, 1);
    } else {
      $colalign['wrap'][$i] = 1;
      $preamble .= strtoupper(substr($align, 0, 1));
    }
  }

  // now, to clean up the table...
  $analyseTableAlign = $colalign;
  $analyseTableY = 0;
  $table = preg_replace_callback("|<row.*?>.*?</row>|s", "analyseRow", $table);

  if ($analyseTableHead && $tableX) {
    $table = preg_replace("|(<tabbody>)\s*(<row.*?>.*?</row>)|s",
			  '<tabhead>$2</tabhead>$1', $table);
  }

  $table = preg_replace("|<tabular(.*?)>|s", 
			'<tabular$1 preamble="'.$preamble.'">', $table);

  return analyseTableKeep($table);
}

function analyseRow($rmatches) {
  global $analyseTableX, $analyseTableY, $analyseTableHead;
  $row = $rmatches[0];
  $analyseTableX = 0;
  if ($analyseTableY == 0) { $analyseTableHead = 0; }
  $row = preg_replace_callback("|<cell(.*?)>(.*?)</cell>|s", 
			       "analyseCell", $row);
  if ($analyseTableY == 0) { $analyseTableHead = $analyseTableHead > 0; }
  $analyseTableY++;
  return $row;
}

function analyseCell($cmatches) {
  global $analyseTableX, $analyseTableAlign, $analyseTableY, $analyseTableHead;
  $cell = $cmatches[0];
  $attr = $cmatches[1];
  $text = $cmatches[2];
  $text = preg_replace("|<[^>]+>|s", "", $text);
  $text = preg_replace("|\\s+|s", "", $text);

  $align = 'left';
  $xwrap = $wrap = 'yes';
  $title = 0+($text == "");

  if (preg_match("|align=['\"](.+?)['\"]|", $attr, $amatches)) {
    $align = $amatches[1];
  }
  $colspan = 1;
  if (preg_match("|colspan=['\"](.+?)['\"]|", $attr, $csmatches)) {
    $colspan = $csmatches[1];
  }
  if (preg_match("|wrap=['\"](.+?)['\"]|", $attr, $wmatches)) {
    $xwrap = $wrap = $wmatches[1];
  }
  if (preg_match("|title=['\"]true['\"]|", $attr, $wmatches)) {
    $title = 1;
    $attr = preg_replace("|title=['\"]true['\"]|", "", $attr);
  }

  // if column is unwrapped, then don't bother specifying alignment.  this makes tabulary happier.
  if ($wrap == 'no' && $align == $analyseTableAlign[$analyseTableX] && $analyseTableAlign['wrap'][$analyseTableX] == 0) {
    $wrap = 'yes';
  }

  if ($wrap == 'yes' && $colspan == 1 && strcmp($align,$analyseTableAlign[$analyseTableX]) == 0) {
    $cell = preg_replace("|<cell(.*?)>|s", "<cell wrap='$xwrap'>", $cell);
  } else {
    $attr = "align='$align' wrap='$xwrap'";
    if ($colspan != 1) $attr .= " colspan='$colspan'";
    $cell = preg_replace("|<cell(.*?)>|s", "<cell $attr>", $cell);
  }

  $analyseTableX += $colspan;
  if ($title && $analyseTableHead >= 0 && $analyseTableY == 0) {
    $analyseTableHead++;
  }
  return $cell;
}

function analyseAdvancedTable($tmatches) {
  global $colCounter;
  $table = '<tabular>' . $tmatches[1]. '</tabular>';
  $colWidths = array();
  preg_match_all("|<row.*?>.*?</row>|s", $table, $rmatches, PREG_SET_ORDER);
  $cols = 0;
  foreach($rmatches as $rm) {
    $x = 0;
    preg_match_all("|<cell(.*?)>.*?</cell>|s", $rm[0], $cmatches, PREG_SET_ORDER);
    foreach($cmatches as $cm) {
      $colspan = 1;
      if (preg_match("|colspan=['\"](.+?)['\"]|", $cm[1], $csmatches)) {
        $colspan = $csmatches[1];
      } elseif (preg_match("|width=['\"](.+?)%['\"]|", $cm[1], $wmatches)) {
        $colWidths[$x] = $wmatches[1];
      }
      $x += $colspan;
      if ($cols < $x) { $cols = $x; }
    }
  }
  $cellwidth = (1.0-(($cols+1)*0.02)) / $cols;

  $align = ""; for ($i=$cols; $i>0; $i--) { $align .= 'l'; }

  $colCounter = 0; 

  return analyseTableKeep(preg_replace(array("|(<cell.*?>)|e",
			    "|(</cell>)|",
			    "|(<tabular)(>)|",
			    "|(<tabbody>)(<row>.*?</row>)(<hline.*?/>)(<row>.*?</row>)|s"),
		      array("'\$1<group class=\"cell\" width=\"' . cellwidthMultiple(PSS('\$1'),\$colWidths,$cols)*$cellwidth . '\">'",
			    "</group>\\1",
			    "\\1 preamble='$align'\\2",
			    "<tabhead>\\2</tabhead>\\1\\3\\4"),
		      $table));

}

function cellwidthMultiple($cell,$colWidths,$cols) {
  global $colCounter;
  if ($colCounter>=$cols) $colCounter = 0;
  $cspan = 1;
  if (preg_match("|colspan=['\"](\d+?)['\"]|", $cell, $cs)) $cspan = $cs[1]; 
  $multiple = 0;
  for($i=$colCounter;$i<$colCounter+$cspan;$i++) 
      $multiple += ($colWidths[$i]) ? $colWidths[$i]*$cols/100 : 1;
  $colCounter += $cspan;
  return $multiple;
}

?>